import numpy as np
from qtpy import QtWidgets


class SmartSlider(QtWidgets.QSlider):
    def __init__(self, *args, min_value=0, max_value=1, step=1, default_value=None, **kwargs):
        super().__init__(*args, **kwargs)

        self._n = int((max_value - min_value) / step) + 1
        self._arr = np.linspace(min_value, max_value, self._n)

        self.default_value = default_value

        self.setRange(1, self._n)
        self.setSingleStep(1)
        self.setValue(self.default_index)

        self._index = self.default_index

    @property
    def default_index(self):
        try:
            return self.index_from_value(self.default_value)
        except (TypeError, ValueError):
            return 1

    @property
    def index(self):
        return self._index

    @index.setter
    def index(self, i):
        self._index = i
        self.setValue(i)

    @property
    def value(self):
        return self._arr[self.index - 1]

    def index_from_value(self, value):
        return self._arr.searchsorted(value, side='right')

    def reset(self):
        self.index = self.default_index



import logging
import pathlib
import yaml
# from dictdiffer import diff, patch, swap
from dataclasses import dataclass, asdict
import dacite
from dacite.exceptions import WrongTypeError, MissingValueError

logger = logging.getLogger(__name__)


@dataclass
class LocalFile:
    directory: str
    filename: str = f"{__package__.split('.')[0]}.yml"
    signature: str = "Local file"

    @property
    def path(self) -> pathlib.Path:
        return pathlib.Path(self.directory) / self.filename

    def save(self, data: dict):
        if not self.path.parent.exists():
            self.path.parent.mkdir()

        if self.path.exists():
            msg = "{} updated (path: {})".format(self.signature, self.path)
        else:
            msg = "{} created (path: {})".format(self.signature, self.path)

        save_yaml(self.path, data)
        logger.debug(msg)


@dataclass
class Params:
    @classmethod
    def read(cls, file: LocalFile, path_to_default: str | None = None):
        default_params = read_yaml(path_to_default, in_dist=True)
        params = dacite.from_dict(data_class=cls, data=default_params)

        if file.path.exists():
            try:
                user_params = read_yaml(file.path)
            except yaml.YAMLError:
                logger.error('Error occurred when parsing `{}`. The file will be overwritten.'.format(file.path))
                cls.save(params, file)
                return params

            # TODO: patch the user config file using dictdiffer
            try:
                user_params = dacite.from_dict(data_class=cls, data=user_params, config=dacite.Config())
            except (WrongTypeError, MissingValueError):
                logger.error('Error occurred when parsing `{}`. The file will be overwritten.'.format(file.path))
                cls.save(params, file)
                return params
            else:
                return user_params
        else:
            cls.save(params, file)

        return params

    def save(self, file: LocalFile):
        file.save(data=asdict(self))


def read_yaml(filename=None, in_dist=False) -> dict:
    if filename is None:
        return {}

    if in_dist:
        yaml_path = (pathlib.Path(__file__).parent.parent / 'data' / filename).resolve()
    else:
        yaml_path = pathlib.Path(filename).resolve()

    with open(yaml_path, "r") as yaml_file:
        return yaml.safe_load(yaml_file)


def filter_none_values(data):
    if isinstance(data, dict):
        return {k: filter_none_values(v) for k, v in data.items() if v is not None}
    return data


def save_yaml(filename, data):
    yaml_path = pathlib.Path(filename).resolve()

    with open(yaml_path, 'w') as yaml_file:
        yaml.dump(data, yaml_file, sort_keys=False)



import logging
import pathlib

from qtpy import QtWidgets

from ..runtime.appdata import AppData
from ..utils import FileBrowser
from ..io.catalogue import load_cat
from ..utils.logs import qlog


logger = logging.getLogger(__name__)


class NewFile(QtWidgets.QDialog):
    def __init__(self, rd: AppData, parent=None):
        self.rd = rd

        super().__init__(parent)
        self.setWindowTitle("Create a New Inspection File")

        layout = QtWidgets.QVBoxLayout()

        self._browsers = {
            'output': FileBrowser(title='Output File:', filename_extensions='CSV Files (*.csv)',
                                  mode=FileBrowser.SaveFile, default_path=pathlib.Path().resolve() / 'Untitled.csv',
                                  parent=self),
            'cat': FileBrowser(title='Catalogue:', filename_extensions='FITS Files (*.fits)', mode=FileBrowser.OpenFile,
                               default_path=self.rd.config.loader.cat.filename, parent=self),
            'data': FileBrowser(title='Data Folder:',  mode=FileBrowser.OpenDirectory,
                                default_path=self.rd.config.loader.data.dir, parent=self)
        }

        for b in self._browsers.values():
            layout.addWidget(b)

        self._filter_check_box = QtWidgets.QCheckBox(
            'Filter the input catalogue based on a list of IDs retrieved from the data folder')
        self._filter_check_box.setChecked(True)
        layout.addWidget(self._filter_check_box)

        self._button_box = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
        layout.addWidget(self._button_box)
        self._button_box.accepted.connect(self.accept)
        self._button_box.rejected.connect(self.reject)

        self.setLayout(layout)

    @qlog
    def process_input(self):
        # validate the input
        for b in self._browsers.values():
            if not b.filled() or not b.exists():
                return

        # load the catalogue
        translate = self.rd.config.loader.cat.translate
        data_folder = self._browsers['data'].path if self._filter_check_box.isChecked() else None

        return load_cat(self._browsers['cat'].path, translate=translate, data_folder=data_folder)

    def accept(self):
        cat = self.process_input()
        if cat is None:
            return

        self.rd.cat = cat
        self.rd.output_path = pathlib.Path(self._browsers['output'].path)

        # update the user configuration file
        self.rd.config.loader.data.dir = self._browsers['data'].path
        self.rd.config.loader.cat.filename = self._browsers['cat'].path
        self.rd.config.save(self.rd.config_file)

        self.rd.create()

        super().accept()



import pathlib
import pandas as pd


def create(ids, checkboxes: dict[str, str] | None) -> pd.DataFrame:
    """ Create a new dataframe containing:
          - a column of IDs;
          - a column for user comments;
          - an extra column per each checkbox.
    @param ids: the list of IDs
    @param checkboxes: the checkboxes to be displayed in the review form
    @return: the dataframe for storing inspection results
    """

    df = pd.DataFrame(index=ids).sort_index()
    df['comment'] = ''

    if checkboxes is not None:
        for i, cname in enumerate(checkboxes.keys()):
            df[cname] = False

    return df


def save(df: pd.DataFrame, filename: str | pathlib.Path):
    """ Save inspection results to the output file.
    @param df: the dataframe containing inspection results
    @param filename: the output filename
    @return: None
    """

    df.to_csv(filename, index_label='id')


def get_checkboxes(df: pd.DataFrame, configured_checkboxes: dict[str, str] | None = None) -> dict[str, str]:
    """ Get checkboxes from the dataframe. By default, each checkbox is automatically assigned a label to be displayed
    in the GUI by a simple capitalization of the column name, e.g. a checkbox named `extended` will get a label
    `Extended`. The `configured_checkboxes` parameter is used to overrides these labels.
    @param df: the dataframe containing inspection results
    @param configured_checkboxes: the checkboxes used to override automatically created labels
    @return: the checkboxes parsed from the dataframe
    """

    if configured_checkboxes is None:
        checkboxes = {}
    else:
        checkboxes = {key: value for key, value in configured_checkboxes.items() if key in df.columns}

    for cname in df.columns:
        if pd.api.types.is_bool_dtype(df[cname]) and cname not in checkboxes.keys():
            checkboxes[cname] = cname.capitalize()

    return checkboxes



import logging
import pathlib
from dataclasses import dataclass
from platformdirs import user_config_dir, user_cache_dir

import pandas as pd
from astropy.table import Table

from ..utils.params import LocalFile
from .config import Config
from .cache import Cache
from ..io import catalogue, output


logger = logging.getLogger(__name__)


@dataclass
class AppData:
    config_file: LocalFile = LocalFile(user_config_dir('specvizitor'), signature='Configuration file')
    cache_file: LocalFile = LocalFile(user_cache_dir('specvizitor'), signature='Cache')

    config: Config = Config.read(config_file, path_to_default='default_config.yml')
    cache: Cache = Cache.read(cache_file)

    output_path: pathlib.Path | None = None  # the path to the output (a.k.a. inspection) file
    cat: Table | None = None                 # the catalogue
    df: pd.DataFrame | None = None           # inspection results

    j: int = None  # the index of the current object

    @property
    def id(self) -> int | None:
        """
        @return: the ID of the current object.
        """
        try:
            return self.df.index[self.j]
        except (TypeError, IndexError):
            return

    @property
    def n_objects(self) -> int | None:
        """
        @return: the total number of objects loaded to the GUI.
        """
        return len(self.df)

    def create(self):
        """ Create a dataframe for storing inspection results.
        """
        if self.cat is None:
            logger.error("Failed to create a new dataframe for storing inspection results: the catalogue is not loaded"
                         "to the memory")
            return

        self.df = output.create(self.cat['id'], self.config.review_form.checkboxes)

    def read(self):
        """ Read the inspection file and load inspection data to the dataframe. If the catalogue hasn't been already
        initialized, load it from the disk. If unsuccessful, create a new catalogue with a single column of IDs given
        by the inspection file.
        """
        if self.output_path is None:
            logger.error("Failed to read the inspection file: the file path is not specified")
            return

        # TODO: validate the input
        df = pd.read_csv(self.output_path, index_col='id')
        df['comment'] = df['comment'].fillna('')

        self.df = df

        if self.cat is None:
            # load the catalogue
            cat = catalogue.load_cat(self.config.loader.cat.filename, translate=self.config.loader.cat.translate)

            # create a catalogue with a single column of IDs
            if cat is None:
                cat = Table([self.df.index.values], names=('id',))
                cat.add_index('id')

            self.cat = cat

    def save(self):
        """ Save inspection results to the output file.
        """
        if self.output_path is None:
            logger.error("Failed to save the inspection results: the output path is not specified")
            return

        output.save(self.df, self.output_path)
        logger.info('Project saved (path: {})'.format(self.output_path))



import logging
from dataclasses import asdict

from qtpy.QtCore import Signal

import numpy as np
from astropy.io import fits
from astropy.utils.decorators import lazyproperty

import pyqtgraph as pg
from qtpy import QtCore, QtWidgets

from ..utils.params import read_yaml
from ..utils import SmartSlider

from .ViewerElement import ViewerElement
from ..runtime.appdata import AppData
from ..runtime import config


logger = logging.getLogger(__name__)


class Spec1D(ViewerElement):
    def __init__(self, rd: AppData, cfg: config.Spectrum, name: str, parent=None):
        super().__init__(rd=rd, cfg=cfg, name=name, parent=parent)

        self.cfg = cfg
        self.name = name

        # load the list of spectral lines
        # TODO: move to the application data
        self._lines = read_yaml('default_lines.yml', in_dist=True)

        # create a label
        self._label = QtWidgets.QLabel()

        # create a widget for the spectrum
        self._spec_1d_widget = pg.GraphicsLayoutWidget()

        # create a redshift slider
        self._z_slider = SmartSlider(QtCore.Qt.Horizontal, **asdict(self.cfg.slider))
        self._z_slider.valueChanged[int].connect(self._update_from_slider)
        self._z_slider.setToolTip('Slide to redshift.')

        # create a line edit for changing the redshift
        self._redshift_editor = QtWidgets.QLineEdit()
        self._redshift_editor.returnPressed.connect(self._update_from_editor)
        self._redshift_editor.setMaximumWidth(120)

        # set up the plot
        self._spec_1d = self._spec_1d_widget.addPlot(name=self.name)
        self._label_style = {'color': 'r', 'font-size': '20px'}

        # set up the spectral lines
        self._line_artists = {}
        # TODO: store colors in config
        line_color = (175.68072, 220.68924, 46.59488)
        line_pen = pg.mkPen(color=line_color, width=1)
        for line_name, lambda0 in self._lines['lambda'].items():
            line = pg.InfiniteLine(angle=90, movable=False, pen=line_pen)
            label = pg.TextItem(text=line_name, color=line_color, anchor=(1, 1), angle=-90)

            self._line_artists[line_name] = {'line': line, 'label': label}

    def init_ui(self):
        self.layout.addWidget(self._label, 1, 1)
        self.layout.addWidget(self._spec_1d_widget, 2, 1, 1, 3)
        self.layout.addWidget(self._z_slider, 3, 1, 1, 1)
        self.layout.addWidget(QtWidgets.QLabel('z = ', self), 3, 2, 1, 1)
        self.layout.addWidget(self._redshift_editor, 3, 3, 1, 1)

    @lazyproperty
    def default_xrange(self):
        return np.nanmin(self._data['wave']), np.nanmax(self._data['wave'])

    @lazyproperty
    def default_yrange(self):
        return np.nanmin(self._data['flux']), np.nanmax(self._data['flux'])

    @lazyproperty
    def _label_height(self):
        y_min, y_max = self.default_yrange
        return y_min + 0.6 * (y_max - y_min)

    def _plot(self):
        self._spec_1d.plot(self._data['wave'], self._data['flux'], pen='k')
        self._spec_1d.plot(self._data['wave'], self._data['err'], pen='r')

        for line_name, line_artist in self._line_artists.items():
            self._spec_1d.addItem(line_artist['line'], ignoreBounds=True)
            self._spec_1d.addItem(line_artist['label'])

        self._spec_1d.setLabel('bottom', self._hdu.header['TUNIT1'], **self._label_style)
        # self._spec_1d.setLabel('left', self._hdu.header['TUNIT2'], **self._label_style)

    def _update_view(self):
        for line_name, line_artist in self._line_artists.items():
            line_wave = self._lines['lambda'][line_name] * (1 + self._z_slider.value)
            line_artist['line'].setPos(line_wave)
            line_artist['label'].setPos(QtCore.QPointF(line_wave, self._label_height))

    def _update_from_slider(self, index=None):
        if index is not None:
            self._z_slider.index = index
        self._redshift_editor.setText("{:.6f}".format(self._z_slider.value))
        self._update_view()

    def _update_from_editor(self):
        try:
            self._z_slider.index = self._z_slider.index_from_value(float(self._redshift_editor.text()))
        except ValueError:
            logger.error('Invalid redshift value: {}'.format(self._redshift_editor.text()))
            self._z_slider.reset()

        self._update_from_slider()

    def reset_view(self):
        if self._data is None:
            return

        self._z_slider.reset()
        self._update_from_slider()

        self._spec_1d.setXRange(*self.default_xrange)
        self._spec_1d.setYRange(*self.default_yrange)

    def load_object(self):
        super().load_object()

        del self.default_xrange
        del self.default_yrange
        del self._label_height

        self._spec_1d.clear()
        if self._data is not None:
            self.setEnabled(True)

            self._label.setText("{}: {}".format(self.name, self._filename.name))

            try:
                self._z_slider.default_value = self.rd.cat.loc[self.rd.id]['z']
            except KeyError:
                self._z_slider.default_value = self.cfg.slider.default_value

            self._plot()
            self.reset_view()
        else:
            self._label.setText("")
            self._redshift_editor.setText("")

            self.setEnabled(False)



from qtpy import QtWidgets

from ..runtime.appdata import AppData
from ..runtime import config
from ..io.output import get_checkboxes
from .AbstractWidget import AbstractWidget


class ReviewForm(QtWidgets.QGroupBox, AbstractWidget):
    def __init__(self, rd: AppData, cfg: config.ReviewForm, parent=None):
        super().__init__(cfg=cfg, parent=parent)

        self.rd = rd
        self.cfg = cfg

        self.setTitle('Review Form')
        self.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)

        # create checkboxes
        self._checkboxes = self.create_checkbox_widgets(self.cfg.checkboxes)

        # create a multi-line text editor for writing comments
        self._comments_widget = QtWidgets.QTextEdit()

    @staticmethod
    def create_checkbox_widgets(checkboxes: dict[str, str] | None) -> dict[str, QtWidgets.QCheckBox]:
        if checkboxes is None:
            return {}

        checkbox_widgets = {}
        for i, (cname, label) in enumerate(checkboxes.items()):
            checkbox_widgets[cname] = QtWidgets.QCheckBox(label)

        return checkbox_widgets

    def init_ui(self):
        for i, widget in enumerate(self._checkboxes.values()):
            self.layout.addWidget(widget, i + 1, 1, 1, 1)

        self.layout.addWidget(QtWidgets.QLabel('Comments:'), len(self._checkboxes) + 1, 1, 1, 1)
        self.layout.addWidget(self._comments_widget, len(self._checkboxes) + 2, 1, 1, 1)

    def dump(self):
        self.rd.df.at[self.rd.id, 'comment'] = self._comments_widget.toPlainText()
        for cname, widget in self._checkboxes.items():
            self.rd.df.at[self.rd.id, cname] = widget.isChecked()

    def load_object(self):
        self._comments_widget.setText(self.rd.df.at[self.rd.id, 'comment'])
        for cname, widget in self._checkboxes.items():
            widget.setChecked(self.rd.df.at[self.rd.id, cname])

    def activate(self):
        self._checkboxes = self.create_checkbox_widgets(get_checkboxes(self.rd.df, self.cfg.checkboxes))
        self.reset_layout()

        super().activate()



from dataclasses import dataclass

from ..utils.params import Params, LocalFile


@dataclass
class Cat:
    filename: str | None
    translate: dict[str, list[str]] | None


@dataclass
class Data:
    dir: str | None


@dataclass
class Loader:
    cat: Cat
    data: Data


@dataclass
class GUI:
    antialiasing: bool = False


@dataclass
class Geometry:
    min_width: int | None
    min_height: int | None


@dataclass
class AbstractWidget(Geometry):
    pass


@dataclass
class ControlPanel(AbstractWidget):
    button_width: int


@dataclass
class ObjectInfo(AbstractWidget):
    items: dict[str, str] | None


@dataclass
class ReviewForm(AbstractWidget):
    checkboxes: dict[str, str] | None


@dataclass
class ViewerElement(AbstractWidget):
    search_mask: str
    link: str | None


@dataclass
class Image(ViewerElement):
    rotate: int | None
    scale: float | None


@dataclass
class Slider:
    min_value: float | None
    max_value: float | None
    step: float | None
    default_value: float | None


@dataclass
class Spectrum(ViewerElement):
    slider: Slider


@dataclass
class Viewer(AbstractWidget):
    images: dict[str, Image]
    spectra: dict[str, Spectrum]


@dataclass
class Config(Params):
    loader: Loader
    writer: None
    gui: GUI
    control_panel: ControlPanel
    object_info: ObjectInfo
    review_form: ReviewForm
    viewer: Viewer
    plugins: list[str]



import logging

# from astropy.coordinates import SkyCoord

from ..runtime.appdata import AppData
from ..runtime import config
from .AbstractWidget import AbstractWidget


from qtpy import QtWidgets, QtCore


logger = logging.getLogger(__name__)


class ObjectInfo(QtWidgets.QGroupBox, AbstractWidget):
    def __init__(self, rd: AppData, cfg: config.ObjectInfo, parent=None):
        super().__init__(cfg=cfg, parent=parent)

        self.rd = rd
        self.cfg = cfg

        self.setTitle('Object Information')
        self.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)

        # display information about the object
        self._labels = self.create_info_widgets(self.cfg.items.items())

    @staticmethod
    def create_info_widgets(items) -> dict[str, QtWidgets.QLabel]:
        if items is None:
            return {}

        info_widgets = {}
        for i, (cname, label) in enumerate(items):
            widget = QtWidgets.QLabel()
            widget.setText(label)
            widget.setHidden(True)
            widget.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse)

            info_widgets[cname] = widget

        return info_widgets

    def init_ui(self):
        for i, widget in enumerate(self._labels.values()):
            self.layout.addWidget(widget, i + 1, 1, 1, 1)

    def load_object(self):
        for cname, widget in self._labels.items():
            try:
                widget.setText(self.cfg.items[cname].format(self.rd.cat.loc[self.rd.id][cname]))
            except KeyError:
                if cname in self.rd.cat.colnames:
                    logger.warning('Object not found in the catalogue (ID: {})'.format(self.rd.id))
                else:
                    logger.warning('`{}` column not found in the catalogue'.format(cname))
                widget.setHidden(True)
            else:
                widget.setHidden(False)

        # if 'ra' in self._cat.colnames and 'dec' in self._cat.colnames:
        #     c = SkyCoord(ra=self._cat['ra'][self._j], dec=self._cat['dec'][self._j], frame='icrs', unit='deg')
        #     ra, dec = c.to_string('hmsdms').split(' ')
        #     self.ra_label.setText("RA: {}".format(ra))
        #     self.dec_label.setText("Dec: {}".format(dec))



from qtpy import QtWidgets


def get_widgets(layout: QtWidgets.QLayout) -> list[QtWidgets.QWidget]:
    widgets = []

    index = layout.count() - 1
    while index >= 0:
        widgets.append(layout.itemAt(index).widget())
        index -= 1

    return widgets



import abc
from abc import ABC

from qtpy import QtWidgets

from ..runtime import config
from ..utils.widgets import get_widgets


class QtAbcMeta(type(QtWidgets.QWidget), type(ABC)):
    pass


class AbstractWidget(QtWidgets.QWidget, ABC, metaclass=QtAbcMeta):
    def __init__(self, cfg: config.AbstractWidget, parent=None):
        super().__init__(parent)

        if cfg.min_width:
            self.setMinimumWidth(cfg.min_width)
        if cfg.min_height:
            self.setMinimumHeight(cfg.min_height)

        self.layout = QtWidgets.QGridLayout()
        self.setLayout(self.layout)

        self.setEnabled(False)

    @abc.abstractmethod
    def init_ui(self):
        pass

    @abc.abstractmethod
    def load_object(self):
        pass

    def reset_layout(self):
        for widget in get_widgets(self.layout):
            self.layout.removeWidget(widget)
            widget.destroy()
        self.init_ui()

    def activate(self):
        self.setEnabled(True)



import importlib

from .AbstractWidget import AbstractWidget
from .ViewerElement import ViewerElement
from .Image2D import Image2D
from .Spec1D import Spec1D

from ..runtime.appdata import AppData
from ..runtime import config
from ..utils.widgets import get_widgets


class DataViewer(AbstractWidget):
    def __init__(self, rd: AppData, cfg: config.Viewer, plugins=None, parent=None):
        super().__init__(cfg=cfg, parent=parent)

        if plugins is not None:
            self._plugins = [importlib.import_module("specvizitor.plugins." + plugin_name).Plugin() for plugin_name in plugins]
        else:
            self._plugins = []

        self.layout.setSpacing(10)
        self.layout.setContentsMargins(0, 0, 0, 0)

        # create widgets for images (image cutout, 2D spectrum, etc.)
        self.images = {}
        for name, image_cfg in cfg.images.items():
            self.images[name] = Image2D(rd=rd, cfg=image_cfg, name=name, parent=self)

        # create widgets for 1D spectra
        self.spectra = {}
        for name, spec_cfg in cfg.spectra.items():
            self.spectra[name] = Spec1D(rd=rd, cfg=spec_cfg, name=name, parent=self)

    def init_ui(self):
        for i, image in enumerate(self.images.values()):
            self.layout.addWidget(image, i + 1, 1, 1, 1)
            image.init_ui()

        for i, spectrum in enumerate(self.spectra.values()):
            self.layout.addWidget(spectrum, i + len(self.images) + 1, 1, 1, 1)
            spectrum.init_ui()

    @property
    def widgets(self) -> list[ViewerElement]:
        """
        @return: a list of widgets added to the data viewer.
        """
        return get_widgets(self.layout)

    def load_object(self):
        for w in self.widgets:
            w.load_object()

        for plugin in self._plugins:
            plugin.link({w.name: w for w in self.widgets})

    def reset_view(self):
        for w in self.widgets:
            w.reset_view()



import pathlib
import argparse
import sys
import logging
from importlib.metadata import version

import pyqtgraph as pg
import qtpy.compat
from qtpy import QtGui, QtWidgets
from qtpy.QtCore import Signal, Slot

from .runtime.appdata import AppData
from .menu import NewFile
from .widgets import (AbstractWidget, DataViewer, ControlPanel, ObjectInfo, ReviewForm)
from .utils.widgets import get_widgets
from .utils.logs import LogMessageBox


logger = logging.getLogger(__name__)


class MainWindow(QtWidgets.QMainWindow):
    project_loaded = Signal()

    def __init__(self, runtime: AppData, parent=None):
        self.rd = runtime

        super().__init__(parent)

        self.setGeometry(600, 500, 2550, 1450)  # set the position and the size of the main window
        self.setWindowTitle('Specvizitor')      # set the title of the main window
        # self.setWindowIcon(QtGui.QIcon('logo2_2.png'))

        # add a menu bar
        self._init_menu()

        # add a status bar
        # self.statusBar().showMessage("Message in statusbar.")

        # create the central widget
        self.central_widget = CentralWidget(self.rd, parent=self)
        self.project_loaded.connect(self.central_widget.activate)
        self.setCentralWidget(self.central_widget)
        self.central_widget.init_ui()

        # read cache and try to load the last active project
        if self.rd.cache.last_inspection_file:
            self.load_project(self.rd.cache.last_inspection_file)

    def _init_menu(self):
        self._menu = self.menuBar()

        self._file = self._menu.addMenu("&File")

        self._new_file = QtWidgets.QAction("&New...")
        self._new_file.triggered.connect(self._new_file_action)
        self._file.addAction(self._new_file)

        self._open_file = QtWidgets.QAction("&Open...")
        self._open_file.triggered.connect(self._open_file_action)
        self._file.addAction(self._open_file)

        self._file.addSeparator()

        self._save = QtWidgets.QAction("&Save...")
        self._save.triggered.connect(self._save_action)
        self._save.setEnabled(False)
        self._file.addAction(self._save)

        self.shortcut_close = QtWidgets.QShortcut(QtGui.QKeySequence('Ctrl+S'), self)
        self.shortcut_close.activated.connect(self._save_action)

        self._save_as = QtWidgets.QAction("Save As...")
        self._save_as.triggered.connect(self._save_as_action)
        self._save_as.setEnabled(False)
        self._file.addAction(self._save_as)

        self._export = QtWidgets.QAction("&Export...")
        self._export.triggered.connect(self._export_action)
        self._export.setEnabled(False)
        self._file.addAction(self._export)

        self._file.addSeparator()

        self._exit = QtWidgets.QAction("E&xit...")
        self._exit.triggered.connect(self._exit_action)
        self._file.addAction(self._exit)

        self._tools = self._menu.addMenu("&Tools")
        self._settings = QtWidgets.QAction("Se&ttings...")
        self._settings.triggered.connect(self._settings_action)
        self._tools.addAction(self._settings)

        self._help = self._menu.addMenu("&Help")
        self._about = QtWidgets.QAction("&About...")
        self._about.triggered.connect(self._about_action)
        self._help.addAction(self._about)

    def _new_file_action(self):
        """ Create a new inspection file via the NewFile dialog.
        """
        dialog = NewFile(self.rd, parent=self)
        if dialog.exec():
            self.rd.cache.last_object_index = 0
            self.activate()

    def _open_file_action(self):
        """ Open an existing inspection file via QFileDialog.
        """
        path = qtpy.compat.getopenfilename(self, caption='Open Inspection File', filters='CSV Files (*.csv)')[0]
        if path:
            self.rd.cache.last_object_index = 0
            self.load_project(path)

    def load_project(self, path: str):
        """ Load inspection data from an existing inspection file.
        @param path: path to the inspection file
        """
        if pathlib.Path(path).exists():
            self.rd.output_path = pathlib.Path(path)
            self.rd.read()
            self.activate()
        else:
            logger.warning('Inspection file not found (path: {})'.format(path))

    def activate(self):
        """ Update the state of the main window and activate the central widget after loading inspection data.
        """
        for w in (self._save, self._save_as, self._export):
            w.setEnabled(True)
        self.setWindowTitle('{} – Specvizitor'.format(self.rd.output_path.name))
        self.project_loaded.emit()

    def _save_action(self):
        """ Instead of saving inspection results, display a message saying that the auto-save mode is enabled.
        """
        msg = 'The data is saved automatically'
        if self.rd.output_path is not None:
            msg += ' to {}'.format(self.rd.output_path)
        LogMessageBox(logging.INFO, msg, parent=self)

    def _save_as_action(self):
        LogMessageBox(logging.INFO, 'Not implemented', parent=self)

    def _export_action(self):
        LogMessageBox(logging.INFO, 'Not implemented', parent=self)

    def _exit_action(self):
        self.rd.save()  # auto-save
        self.close()
        logger.info("Application closed")

    def _settings_action(self):
        QtWidgets.QMessageBox.information(self, "Settings",
                                          "Configuration file: {}\n\nCache: {}".
                                          format(self.rd.config_file.path, self.rd.cache_file.path))

    def _about_action(self):
        QtWidgets.QMessageBox.about(self, "About Specvizitor", "Specvizitor v{}".format(version('specvizitor')))

    def closeEvent(self, _):
        self._exit_action()


class CentralWidget(QtWidgets.QWidget):
    def __init__(self, rd: AppData, parent=None):
        self.rd = rd
        super().__init__(parent)

        # set up the layout
        self.layout = QtWidgets.QGridLayout()
        self.layout.setSpacing(10)
        self.setLayout(self.layout)

        # add a widget for the data viewer
        self.data_viewer = DataViewer(self.rd, cfg=self.rd.config.viewer, plugins=self.rd.config.plugins, parent=self)
        self.layout.addWidget(self.data_viewer, 1, 1, 3, 1)

        # add a widget for the control panel
        self.control_panel = ControlPanel(self.rd, cfg=self.rd.config.control_panel, parent=self)
        self.layout.addWidget(self.control_panel, 1, 2, 1, 1)

        # add a widget for displaying information about the object
        self.object_info = ObjectInfo(self.rd, cfg=self.rd.config.object_info, parent=self)
        self.layout.addWidget(self.object_info, 2, 2, 1, 1)

        # add a widget for writing comments
        self.review_form = ReviewForm(self.rd, cfg=self.rd.config.review_form, parent=self)
        self.layout.addWidget(self.review_form, 3, 2, 1, 1)

        # connect signals from the control panel to the slots of the central widget
        self.control_panel.object_selected.connect(self.load_object)
        self.control_panel.reset_button_clicked.connect(self.data_viewer.reset_view)

    @property
    def widgets(self) -> list[AbstractWidget]:
        """
        @return: a list of widgets added to the central widget.
        """
        return get_widgets(self.layout)

    def init_ui(self):
        for widget in self.widgets:
            widget.init_ui()

    @Slot(int)
    def load_object(self, j: int):
        """ Load a new object to the central widget.
        @param j: the index of the object to display
        """
        if self.rd.j is not None:
            # update the application data from widgets
            self.review_form.dump()

            self.rd.save()  # auto-save

        self.rd.j = j

        for widget in self.widgets:
            widget.load_object()

        # cache the index of the object
        # TODO: cache the ID instead of the index
        self.rd.cache.last_object_index = j
        self.rd.cache.save(self.rd.cache_file)

    @Slot()
    def activate(self):
        """ Activate the central widget.
        """
        for widget in self.widgets:
            widget.activate()

        # cache the inspection file name
        self.rd.cache.last_inspection_file = str(self.rd.output_path)
        self.rd.cache.save(self.rd.cache_file)

        # try to display the object with an index stored in cache
        j = self.rd.cache.last_object_index
        if j and j < self.rd.n_objects:
            self.load_object(int(j))
        else:
            self.load_object(0)


def main():
    parser = argparse.ArgumentParser()

    # logging configuration
    parser.add_argument('-v', '--verbose', action='store_true')
    args = parser.parse_args()

    level = logging.DEBUG if args.verbose else logging.WARNING
    logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=level)

    # initialize the application data
    runtime = AppData()

    # pyqtgraph configuration
    pg.setConfigOption('background', 'w')
    pg.setConfigOption('foreground', 'k')
    pg.setConfigOption('antialias', runtime.config.gui.antialiasing)

    # start the application
    app = QtWidgets.QApplication(sys.argv)
    logger.info("Application started")

    # initialize the main window
    window = MainWindow(runtime=runtime)
    window.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



import logging
import pathlib

logger = logging.getLogger(__name__)


def get_filename(directory, search_mask: str, object_id) -> pathlib.Path | None:
    """ Find a file for a given search mask and object ID. If more than one filename is matched to the search mask,
    return the first item from an alphabetically ordered list of matched filenames.

    @param directory: the directory where the search is performed
    @param search_mask: the search mask
    Must contain `{}` to indicate the place for ID insertion. Examples: `{}.fits`, `*{}.1D.fits`, `*{}*`
    @param object_id: the object ID
    @return: the file name matched to the search mask
    """

    if '{}' not in search_mask:
        logger.error('Invalid search mask: `{}`. The search mask must contain `{{}}` to indicate the place for ID '
                     'insertion'.format(search_mask))
        return

    matched_filenames = sorted(list(pathlib.Path(directory).glob(search_mask.format(object_id))))

    if matched_filenames:
        return matched_filenames[0]
    else:
        return


# TODO: create a FilenameParser class
def get_grizli_id(filename) -> int | None:
    """ Parse a file name to get the object ID. The file must be one of the grizli fit products.
    @param filename: the file name to parse
    @return: the object ID
    """

    try:
        return int(pathlib.Path(filename).name.split('_')[1].split('.')[0])
    except (IndexError, ValueError):
        # commented out because might be called too many times
        # logger.error('Failed to parse the file name `{}`'.format(filename))
        return



import logging

import numpy as np
import pyqtgraph as pg
from astropy.visualization import ZScaleInterval
from astropy.utils.decorators import lazyproperty

from qtpy import QtWidgets
from pgcolorbar.colorlegend import ColorLegendItem

from .ViewerElement import ViewerElement
from ..runtime.appdata import AppData
from ..runtime import config


logger = logging.getLogger(__name__)


class Image2D(ViewerElement):
    def __init__(self, rd: AppData, cfg: config.Image, name: str, parent=None):
        super().__init__(rd=rd, cfg=cfg, name=name, parent=parent)

        self.cfg = cfg
        self.name = name

        # add a label
        self._label = QtWidgets.QLabel()

        # add a widget for the spectrum
        self._image_2d_widget = pg.GraphicsLayoutWidget()

        # set up the color map
        self._cmap = pg.colormap.get('viridis')

        # set up the image and the view box
        self.image_2d_plot = self._image_2d_widget.addPlot(name=name)

        self._image_2d = pg.ImageItem(border='k')
        self._image_2d.setLookupTable(self._cmap.getLookupTable())
        self.image_2d_plot.addItem(self._image_2d)

        # set up the color bar
        self._cbar = ColorLegendItem(imageItem=self._image_2d, showHistogram=True, histHeightPercentile=99.0)
        self._image_2d_widget.addItem(self._cbar, 0, 1)

        # lock the aspect ratio
        self.image_2d_plot.setAspectLocked(True)

    def init_ui(self):
        self.layout.addWidget(self._label, 1, 1)
        self.layout.addWidget(self._image_2d_widget, 2, 1)

    @lazyproperty
    def _data(self):
        data = super()._data
        if data is None:
            return

        # rotate the image
        if self.cfg.rotate is not None:
            data = np.rot90(data, k=self.cfg.rotate // 90)

        # scale the data points
        if self.cfg.scale is not None:
            data *= self.cfg.scale

        return data

    def reset_view(self):
        if self._data is None:
            return

        # TODO: allow to choose between min/max and zscale?
        self._cbar.setLevels(ZScaleInterval().get_limits(self._data))

        self.image_2d_plot.autoRange()

    def load_object(self):
        super().load_object()

        if self._data is not None:
            self.setEnabled(True)

            self._label.setText("{}: {}".format(self.name, self._filename.name))
            self._image_2d.setImage(self._data)

            self.reset_view()
        else:
            self._label.setText("")
            self._image_2d.clear()

            self.setEnabled(False)



from qtpy import QtGui

from ..widgets.ViewerElement import ViewerElement


class Plugin:
    # TODO: create BasePlugin
    @staticmethod
    def link(widgets: dict[str, ViewerElement]):

        try:
            spec_1d, spec_2d = widgets["Spectrum 1D"], widgets["Spectrum 2D"]
        except KeyError:
            return

        if any(w._data is None for w in (spec_1d, spec_2d)):
            return

        # set x-axis transformation for the 2D spectrum plot
        DLAM = spec_2d._hdu.header['CD1_1'] * 1e4
        CRVAL = spec_2d._hdu.header['CRVAL1'] * 1e4
        CRPIX = spec_2d._hdu.header['CRPIX1']
        qtransform = QtGui.QTransform(DLAM, 0, 0,
                                      0, 1, 0,
                                      CRVAL - DLAM * CRPIX, 0, 1)

        spec_2d._image_2d.setTransform(qtransform)
        spec_2d.image_2d_plot.setAspectLocked(True, 1 / DLAM)
        spec_2d.image_2d_plot.setXLink(spec_1d.name)  # link the x-axis range



import logging
import pathlib

import numpy as np
from astropy.io import fits
from astropy.table import Table

from .viewer_data import get_grizli_id

logger = logging.getLogger(__name__)


def load_cat(filename=None,
             translate: dict[str, list[str]] | None = None,
             data_folder=None,
             filename_parser=get_grizli_id) -> [Table, None]:

    """ Read and filter the catalogue.
    @param filename: the catalogue filename
    @param translate:
    @param data_folder:
    @param filename_parser:
    @return: the processed catalogue
    """

    if filename is None:
        logger.warning('Catalogue filename not specified')
        return

    # load the catalogue
    try:
        cat = Table(fits.getdata(filename))
    except OSError:
        logger.error('Failed to load the catalogue')
        return

    # rename columns
    if translate is not None:
        for cname, cname_synonyms in translate.items():
            for syn in cname_synonyms:
                if syn in cat.colnames:
                    cat.rename_column(syn, cname)
                    break

    # check that the ID column is present in the catalogue
    if 'id' not in cat.colnames:
        logger.error(column_not_found_message('id', translate))
        return
    cat.add_index('id')

    if data_folder is not None:
        # retrieve IDs from the data folder
        data_files = sorted(pathlib.Path(data_folder).glob('**/*.fits'))
        obj_ids = [filename_parser(p) for p in data_files]
        obj_ids = np.unique([id_ for id_ in obj_ids if id_ is not None])

        if not obj_ids.size:
            logger.error('No IDs retrieved from the data folder')
            return

        # filter objects based on the list of IDs
        cat = cat[np.in1d(cat['id'], obj_ids, assume_unique=True)]

    if len(cat) == 0:
        logger.error('The processed catalogue is empty')
        return

    return cat


def column_not_found_message(cname: str, translate: dict[str, list[str]] | None = None) -> str:
    """ Create the `column not found` message.
    @param cname: the column name
    @param translate:
    @return: the message
    """

    if translate is None or cname not in translate:
        return '`{}` column not found'.format(cname)
    else:
        return '`{}` column and its equivalences ({}) not found'.format(cname, ", ".join(translate[cname]))



import logging
from functools import partial

from qtpy import QtCore, QtWidgets

from ..runtime.appdata import AppData
from ..runtime import config
from .AbstractWidget import AbstractWidget


logger = logging.getLogger(__name__)


class ControlPanel(QtWidgets.QGroupBox, AbstractWidget):
    reset_button_clicked = QtCore.Signal()
    object_selected = QtCore.Signal(int)

    def __init__(self, rd: AppData, cfg: config.ControlPanel, parent=None):
        super().__init__(cfg=cfg, parent=parent)

        self.rd = rd
        self.cfg = cfg

        self.setTitle('Control Panel')
        self.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)

        # create a reset button
        self._reset_button = QtWidgets.QPushButton()
        self._reset_button.setText('ID --')
        self._reset_button.setToolTip('Reset the view')
        self._reset_button.setFixedWidth(self.cfg.button_width)
        self._reset_button.clicked.connect(self.reset_button_clicked.emit)

        # create a widget displaying the index of the current object and the total number of objects in the catalogue
        self._number_of_obj_label = QtWidgets.QLabel()

        # create buttons for switching to the next or previous object
        self._pn_buttons = self.create_pn_buttons()

        # create a `Go to ID` button
        self._go_to_id_button = QtWidgets.QPushButton()
        self._go_to_id_button.setText('Go to ID')
        self._go_to_id_button.setFixedWidth(self.cfg.button_width)
        self._go_to_id_button.clicked.connect(self.go_to_id)

        self._id_field = QtWidgets.QLineEdit()
        self._id_field.setFixedWidth(self.cfg.button_width)
        self._id_field.returnPressed.connect(self.go_to_id)

        # create a `Go to index` button
        self._go_to_index_button = QtWidgets.QPushButton()
        self._go_to_index_button.setText('Go to #')
        self._go_to_index_button.setFixedWidth(self.cfg.button_width)
        self._go_to_index_button.clicked.connect(self.go_to_index)

        self._index_field = QtWidgets.QLineEdit()
        self._index_field.setFixedWidth(self.cfg.button_width)
        self._index_field.returnPressed.connect(self.go_to_index)

    def create_pn_buttons(self) -> dict[str, QtWidgets.QPushButton]:
        pn_buttons_params = {'previous': {'shortcut': 'left'},
                             'next': {'shortcut': 'right'}}

        pn_buttons = {}
        for pn_text, pn_properties in pn_buttons_params.items():
            button = QtWidgets.QPushButton('')
            # button.setIcon(QtGui.QIcon(pn_text + '.png'))
            button.setToolTip('Look at the {} object'.format(pn_text))
            button.setText(pn_text)
            button.setFixedWidth(self.cfg.button_width)
            button.clicked.connect(partial(self.previous_next_object, pn_text))
            button.setShortcut(pn_properties['shortcut'])

            pn_buttons[pn_text] = button

        return pn_buttons

    def init_ui(self):
        self.layout.addWidget(self._reset_button, 1, 1, 1, 1)
        self.layout.addWidget(self._number_of_obj_label, 1, 2, 1, 1)

        self.layout.addWidget(self._pn_buttons['previous'], 2, 1, 1, 1)
        self.layout.addWidget(self._pn_buttons['next'], 2, 2, 1, 1)

        self.layout.addWidget(self._go_to_id_button, 3, 1, 1, 1)
        self.layout.addWidget(self._id_field, 3, 2, 1, 1)
        self.layout.addWidget(self._go_to_index_button, 4, 1, 1, 1)
        self.layout.addWidget(self._index_field, 4, 2, 1, 1)

    def load_object(self):
        self._reset_button.setText('ID {}'.format(self.rd.id))
        self._number_of_obj_label.setText('(#{} / {})'.format(self.rd.j + 1, self.rd.n_objects))

    def previous_next_object(self, command: str):
        j_upd = self.rd.j

        if command == 'next':
            j_upd += 1
        elif command == 'previous':
            j_upd -= 1

        j_upd = j_upd % self.rd.n_objects

        self.object_selected.emit(j_upd)

    def go_to_id(self):
        text = self._id_field.text()
        self._id_field.clear()

        try:
            id_upd = int(text)
        except ValueError:
            logger.error('Invalid ID')
            return

        if id_upd in self.rd.df.index:
            self.object_selected.emit(self.rd.df.index.get_loc(id_upd))
        else:
            logger.error('ID not found')
            return

    def go_to_index(self):
        text = self._index_field.text()
        self._index_field.clear()

        try:
            index_upd = int(text)
        except ValueError:
            logger.error('Invalid index')
            return

        if 0 < index_upd <= self.rd.n_objects:
            self.object_selected.emit(index_upd - 1)
        else:
            logger.error('Index out of range')
            return



import logging
import pathlib

import qtpy.compat
from qtpy import QtWidgets

logger = logging.getLogger(__name__)


class FileBrowser(QtWidgets.QWidget):
    OpenFile = 0
    # OpenFiles = 1
    OpenDirectory = 2
    SaveFile = 3

    def __init__(self, title=None, mode=OpenFile, filename_extensions='All files (*.*)', default_path=None,
                 button_text='Browse...', parent=None):
        super().__init__(parent=parent)
        self._parent = parent

        self._title = title
        self._browser_mode = mode
        self._filter = filename_extensions  # example: 'Images (*.png *.xpm *.jpg);;Text files (*.txt)'
        self._default_path = None if default_path is None else str(pathlib.Path(default_path).resolve())

        layout = QtWidgets.QHBoxLayout()

        self._label = QtWidgets.QLabel(title)
        self._label.setFixedWidth(120)
        layout.addWidget(self._label)

        self._line_edit = QtWidgets.QLineEdit(self)
        self._line_edit.setMinimumWidth(700)
        self._line_edit.setText(self._default_path)
        layout.addWidget(self._line_edit)

        self._button = QtWidgets.QPushButton(button_text)
        self._button.setFixedWidth(120)
        self._button.clicked.connect(self._browse)
        layout.addWidget(self._button)

        self.setLayout(layout)

    @property
    def path(self):
        return self._line_edit.text()

    def filled(self):
        if not self.path:
            logger.error('The field `{}` is empty'.format(self._title.rstrip(':')))
            return False
        return True

    def exists(self):
        if self._browser_mode == FileBrowser.SaveFile:
            path_to_check = pathlib.Path(self.path).parent
        else:
            path_to_check = pathlib.Path(self.path)

        if not path_to_check.exists():
            if self._browser_mode == FileBrowser.OpenFile:
                msg = 'The file `{}` does not exist'.format(path_to_check)
            elif self._browser_mode in (FileBrowser.OpenDirectory, FileBrowser.SaveFile):
                msg = 'The directory `{}` does not exist'.format(path_to_check)
            else:
                msg = 'The path `{}` does not exist'.format(path_to_check)

            logger.error(msg)
            return False

        return True

    def _browse(self):
        path = None

        if self._browser_mode == FileBrowser.OpenFile:
            path = qtpy.compat.getopenfilename(self, caption='Choose File', basedir=self._default_path,
                                               filters=self._filter)[0]

        # elif self._browser_mode == FileBrowser.OpenFiles:
        #     self._filepaths.extend(qtpy.compat.getopenfilenames(self, caption='Choose Files',
        #                                                                   basedir=self._default_path,
        #                                                                   filters=self._filter)[0])

        elif self._browser_mode == FileBrowser.OpenDirectory:
            path = qtpy.compat.getexistingdirectory(self, caption='Choose Directory', basedir=self._default_path)

        elif self._browser_mode == FileBrowser.SaveFile:
            options = QtWidgets.QFileDialog.Options()
            # if sys.platform == 'darwin':
            #    options |= QtWidgets.QFileDialog.DontUseNativeDialog
            path, extension = qtpy.compat.getsavefilename(self, caption='Save/Save As', basedir=self._default_path,
                                                          filters=self._filter, options=options)

        if path:
            self._line_edit.setText(path)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['specvizitor',
 'specvizitor.io',
 'specvizitor.menu',
 'specvizitor.plugins',
 'specvizitor.runtime',
 'specvizitor.utils',
 'specvizitor.widgets']

package_data = \
{'': ['*'], 'specvizitor': ['data/*']}

install_requires = \
['astropy>=5.2.1,<6.0.0',
 'dacite>=1.8.0,<2.0.0',
 'dictdiffer>=0.9.0,<0.10.0',
 'pandas>=1.5.3,<2.0.0',
 'pgcolorbar>=1.1.3,<2.0.0',
 'platformdirs>=3.0.0,<4.0.0',
 'pyqt5>=5.15.9,<6.0.0',
 'pyqtgraph>=0.13.1,<0.14.0',
 'qtpy>=2.3.0,<3.0.0']

entry_points = \
{'console_scripts': ['specvizitor = specvizitor.gui:main']}

setup_kwargs = {
    'name': 'specvizitor',
    'version': '0.1.10',
    'description': 'Python GUI application for a visual inspection of astronomical spectroscopic data',
    'long_description': '![Specvizitor GUI](https://github.com/ivkram/specvizitor/blob/main/docs/screenshots/specvizitor_gui.png?raw=true "Specvizitor GUI")\n\n## Installation\n\n### Installing `specvizitor` using pip\n\nSet up a local environment and run\n\n        $ pip install specvizitor\n\n### Installing `specvizitor` from source\n\n1. Clone the public repository:\n\n        $ git clone https://github.com/ivkram/specvizitor\n        $ cd specvizitor\n\n2. Set up a local environment and run\n\n        $ pip install -e .\n\n## Starting `specvizitor`\n    \nTo start `specvizitor`, run this command in your terminal:    \n\n    $ specvizitor\n\n## License\n\n`specvizitor` is licensed under a 3-clause BSD style license - see the [LICENSE.txt](https://github.com/ivkram/specvizitor/blob/main/LICENSE.txt) file.',
    'author': 'Ivan Kramarenko',
    'author_email': 'im.kramarenko@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ivkram/specvizitor',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



# import eazy

from qtpy import QtWidgets


class Eazy(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setEnabled(False)

        grid = QtWidgets.QGridLayout()

        self._run_eazy_button = QtWidgets.QPushButton("Run Eazy")
        self._run_eazy_button.clicked.connect(self.run_eazy)
        grid.addWidget(self._run_eazy_button, 1, 1)

        '''
        z_raw_chi2 = np.round(self.zout['z_raw_chi2'][self.j],3)
        eazy_raw_chi2 = QtWidgets.QLabel('Chi2: '+str(z_raw_chi2), self)

        raw_chi2 = np.round(self.zout['raw_chi2'][self.j],3)
        eazy_raw_chi2 = QtWidgets.QLabel('Chi2: '+str(raw_chi2), self)
        grid.addWidget(eazy_raw_chi2,8,31,1,1)

        self.z_phot_chi2 = np.round(self.zout['z_phot_chi2'][self.j], 3)
        eazy_raw_chi2 = QtWidgets.QLabel('Chi2: ' + str(self.z_phot_chi2), self)

        self.sfr = np.round(self.zout['sfr'][self.j], 3)
        eazy_sfr = QtWidgets.QLabel('SFR: ' + str(self.sfr), self)

        self.mass = np.round(self.zout['mass'][self.j] / 10 ** 9, 3)
        eazy_mass = QtWidgets.QLabel('mass: ' + str(self.mass), self)
        '''

        self.setLayout(grid)

    def run_eazy(self):
        pass
        # eazy_inst = eazy.photoz.PhotoZ(param_file=self._parent.config['data']['eazy']['param_file'],
        #                                translate_file=self._parent.config['data']['eazy']['translate_file'],
        #                                zeropoint_file=self._parent.config['data']['eazy']['zeropoint_file'],
        #                                load_prior=False, load_products=False, n_proc=-1)



import logging
import abc

from astropy.utils import lazyproperty
from astropy.io import fits

from .AbstractWidget import AbstractWidget

from ..runtime.appdata import AppData
from ..runtime import config
from ..io.viewer_data import get_filename


logger = logging.getLogger(__name__)


class ViewerElement(AbstractWidget, abc.ABC):
    def __init__(self, rd: AppData, cfg: config.ViewerElement, name: str, parent=None):
        super().__init__(cfg=cfg, parent=parent)

        self.rd = rd
        self.cfg = cfg
        self.name = name

        self.layout.setContentsMargins(0, 0, 0, 0)

    @lazyproperty
    def _filename(self):
        return get_filename(self.rd.config.loader.data.dir, self.cfg.search_mask, self.rd.id)

    @lazyproperty
    def _hdul(self):
        try:
            hdul = fits.open(self._filename)
        except ValueError:
            logger.warning('{} not found (object ID: {})'.format(self.name, self.rd.id))
            return
        else:
            return hdul

    @lazyproperty
    def _hdu(self):
        return self._hdul[1] if self._hdul else None

    @lazyproperty
    def _data(self):
        return self._hdu.data if self._hdu is not None else None

    @abc.abstractmethod
    def reset_view(self):
        pass

    def load_object(self):
        del self._filename
        del self._hdul
        del self._hdu
        del self._data
